# Generated by Django 3.0.6 on 2020-12-19 22:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_scoped_permissions', '0002_scopedpermissiongroup'),
        ('tests', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='scoped_permission_groups',
            field=models.ManyToManyField(blank=True, to='django_scoped_permissions.ScopedPermissionGroup'),
        ),
        migrations.AddField(
            model_name='usertype',
            name='scoped_permission_groups',
            field=models.ManyToManyField(blank=True, to='django_scoped_permissions.ScopedPermissionGroup'),
        ),
    ]
